@@ -1,14 +1,13 @@
 """The 'batch' command."""
-from __future__ import absolute_import, division, print_function
-
 import logging
 import os
 
 from matplotlib import pyplot
 from skgenome import tabio, GenomicArray as GA
 
-from . import (access, antitarget, autobin, core, coverage, diagram, fix,
-               parallel, reference, scatter, segmentation, target)
+from . import (access, antitarget, autobin, bintest, call, core, coverage,
+               diagram, fix, parallel, reference, scatter, segmentation,
+               segmetrics, target)
 from .cmdutil import read_cna
 
 
@@ -16,7 +15,7 @@ def batch_make_reference(normal_bams, target_bed, antitarget_bed,
                          male_reference, fasta, annotate, short_names,
                          target_avg_size, access_bed, antitarget_avg_size,
                          antitarget_min_size, output_reference, output_dir,
-                         processes, by_count, method):
+                         processes, by_count, method, do_cluster):
     """Build the CN reference from normal samples, targets and antitargets."""
     if method in ("wgs", "amplicon"):
         if antitarget_bed:
@@ -43,7 +42,7 @@ def batch_make_reference(normal_bams, target_bed, antitarget_bed,
                 access_arr = access.do_access(fasta)
                 # Take filename base from FASTA, lacking any other clue
                 target_bed = os.path.splitext(os.path.basename(fasta)
-                                             )[0] + ".bed"
+                                              )[0] + ".bed"
                 tabio.write(access_arr, target_bed, "bed3")
             else:
                 raise ValueError("WGS protocol: need to provide --targets, "
@@ -144,7 +143,8 @@ def batch_make_reference(normal_bams, target_bed, antitarget_bed,
                                          fasta, male_reference, None,
                                          do_gc=True,
                                          do_edge=(method == "hybrid"),
-                                         do_rmask=True)
+                                         do_rmask=True,
+                                         do_cluster=do_cluster)
     if not output_reference:
         output_reference = os.path.join(output_dir, "reference.cnn")
     core.ensure_path(output_reference)
@@ -161,7 +161,8 @@ def batch_write_coverage(bed_fname, bam_fname, out_fname, by_count, processes):
 
 def batch_run_sample(bam_fname, target_bed, antitarget_bed, ref_fname,
                      output_dir, male_reference, plot_scatter, plot_diagram,
-                     rscript_path, by_count, skip_low, method, processes):
+                     rscript_path, by_count, skip_low, seq_method,
+                     segment_method, processes, do_cluster):
     """Run the pipeline on one BAM file."""
     # ENH - return probes, segments (cnarr, segarr)
     logging.info("Running the CNVkit pipeline on %s ...", bam_fname)
@@ -177,29 +178,48 @@ def batch_run_sample(bam_fname, target_bed, antitarget_bed, ref_fname,
     tabio.write(raw_anti, sample_pfx + '.antitargetcoverage.cnn')
 
     cnarr = fix.do_fix(raw_tgt, raw_anti, read_cna(ref_fname),
-                       do_gc=True, do_edge=(method == "hybrid"), do_rmask=True)
+                       do_gc=True, do_edge=(seq_method == "hybrid"), do_rmask=True,
+                       do_cluster=do_cluster)
     tabio.write(cnarr, sample_pfx + '.cnr')
 
     logging.info("Segmenting %s.cnr ...", sample_pfx)
-    segments = segmentation.do_segmentation(cnarr, 'cbs',
+    segments = segmentation.do_segmentation(cnarr, segment_method,
                                             rscript_path=rscript_path,
                                             skip_low=skip_low,
                                             processes=processes,
                                             **({'threshold': 1e-6}
-                                               if method == 'wgs'
+                                               if seq_method == 'wgs'
                                                else {}))
-    tabio.write(segments, sample_pfx + '.cns')
+    logging.info("Post-processing %s.cns ...", sample_pfx)
+    # TODO/ENH take centering shift & apply to .cnr for use in segmetrics
+    seg_metrics = segmetrics.do_segmetrics(cnarr, segments,
+                                           interval_stats=['ci'], alpha=0.5,
+                                           smoothed=True)
+    tabio.write(seg_metrics, sample_pfx + '.cns')
+
+    # Remove likely false-positive breakpoints
+    seg_call = call.do_call(seg_metrics, method="none", filters=['ci'])
+    # Calculate another segment-level test p-value
+    seg_alltest = segmetrics.do_segmetrics(cnarr, seg_call, location_stats=['p_ttest'])
+    # Finally, assign absolute copy number values to each segment
+    seg_alltest.center_all("median")
+    seg_final = call.do_call(seg_alltest, method="threshold")
+    tabio.write(seg_final, sample_pfx + '.call.cns')
+
+    # Test for single-bin CNVs separately
+    seg_bintest = bintest.do_bintest(cnarr, seg_call, target_only=True)
+    tabio.write(seg_bintest, sample_pfx + '.bintest.cns')
 
     if plot_scatter:
-        scatter.do_scatter(cnarr, segments)
-        pyplot.savefig(sample_pfx + '-scatter.pdf', format='pdf',
+        scatter.do_scatter(cnarr, seg_final)
+        pyplot.savefig(sample_pfx + '-scatter.png', format='png',
                        bbox_inches="tight")
-        logging.info("Wrote %s-scatter.pdf", sample_pfx)
+        logging.info("Wrote %s-scatter.png", sample_pfx)
 
     if plot_diagram:
         is_xx = cnarr.guess_xx(male_reference)
         outfname = sample_pfx + '-diagram.pdf'
         diagram.create_diagram(cnarr.shift_xx(male_reference, is_xx),
-                               segments.shift_xx(male_reference, is_xx),
+                               seg_final.shift_xx(male_reference, is_xx),
                                0.5, 3, outfname)
         logging.info("Wrote %s", outfname)
